% This module encodes the first-order representation of the 
% knowledge representation primtives in the ACL86 paper.

module fol.

%  This are the primitive types
kind i       type.
kind concept type.
kind role    type.

%     The following are used to build concepts
type and      concept -> concept -> concept.
type restr    role -> concept -> concept.

%     These are the program predicates
type role     role -> o.
type concept  concept -> o.
type subsume  concept -> concept -> o.
type is_a     concept -> i -> o.
type related  role -> i -> i -> o.

%  The higher-order predicates are "compiled" using the following:
%	and 	:= C\D\X\(C X & D X)
%	restr	:= R\C\X\(all Y\(R X Y => C Y))
%	is_a	:= C\X\(fact (C X))
%	related	:= R\X\Y\(fact (R X Y))

role R :- prim_role R.
concept C :- prim_concept C.
concept (and C1 C2) :- concept C1, concept C2.
concept (restr R C1) :- concept C1, role R.

%  These clauses define the logic of subsume 
subsume C C.
subsume A B :- subsume A C, subsume C B.
subsume (and A B) C :- subsume A C, subsume B C.
subsume A (and B C) :- subsume A B.
subsume A (and B C) :- subsume A C.
subsume (restr R A) (restr R B) :- subsume A B.

%  These clauses provide the logic of is_a.
is_a (and A B) C :- is_a A C, is_a B C.
is_a C U :- subsume (restr R C) D, related R V U, is_a D V.
is_a C X :- subsume C D, is_a D X.

%  The following clauses represent the state of the world, i.e. they
%  are a small database.
is_a important_message m1.
related sender m1 kirk.
related recipient m1 scotty.
subsume person crew.
subsume (restr sender person) message.
subsume (restr recipient crew) message.
subsume message important_message.
subsume (restr sender commander) important_message.
