% A module that encodes knowledge about the fictional world of STARTREK.

module	startrek.

import  kltypes.

% These can be inferred, but provide handy documentation
type message      i -> bool.
type person       i -> bool.
type crew         i -> bool.
type recipient    i -> i -> bool.
type body         i -> i -> bool.
type sender       i -> i -> bool.
type senddate     i -> i -> bool.
type receivedate  i -> i -> bool.


%%  Some simple facts

%		membership in concepts (is_a statements)
fact (message m1).
fact (person kirk).
fact (crew sulu).
fact (crew scotty).

%		 relations among individuals
fact (recipient m1 sulu).
fact (recipient m1 scotty).
fact (recipient m1 uhura).
fact (body m1 send-help).
fact (sender kirk m1).

%   some primitive concepts
prim_concept message.
prim_concept person.
prim_concept crew.
prim_concept commander.
prim_concept important_message.

%   some primitive roles
prim_role recipient.
prim_role body.
prim_role sender.
prim_role senddate.
prim_role receivedate.

%%  Some subsumption relations

%	 for messages
prim_subsume thing					 message.
prim_subsume (X\ (all Y\ (senddate X Y ==> date Y)))     message.
prim_subsume (X\ (all Y\ (receiveddate X Y ==> date Y))) message.
prim_subsume (X\ (all Y\ (sender X Y ==> person Y)))     message.
prim_subsume (X\ (all Y\ (recipient X Y ==> person Y)))	 message.
prim_subsume (X\ (all Y\ (body X Y ==> text Y)))       	 message.
prim_subsume (X\ (all Y\(recipient X Y ==> crew Y)))	 message.

%	 for crew
prim_subsume person crew.

%	 for commander
prim_subsume person commander.

%	 for starfleet_message
prim_subsume message                                  	starfleet_message.
prim_subsume (X\ (all Y\ (sender X Y ==> commander Y)))	starfleet_message.

%	 for important messages
prim_subsume message important_message.
prim_subsume (X\ (all Y\(sender X Y ==> commander Y))) important_message.