%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%	CAM : Categorical Abstract Machine		  %%%
%%%	Test Cases for evaluator			  %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

module camtest.

import cameval.

%%% This file contains the compiled CAM code for the mini-ML examples
%%% in the file cmltest.  Following the code are some simple procedures
%%% for exercising this code.
%%% N.B. These programs need a fairly large global stack to run.  I have
%%% used 40000K on faulkner.  These programs also take a *long* time to
%%% run, so you might want to start them up and then go out for coffee.
%%% (The "cammap" procedure requires a global stack of size 47000K.  If
%%% you run it, better go out for lunch, not just coffee!)

%-----------------------------------------------------------------------%

%%% CAM code for the factorial function (from 5.2 of Kahn's STACS paper)

factprog N 
  (program
  	(push &
	(rec  (cur (push &
			 push & csnd &
			 swap & (quote (## 0)) &
			 cons & (op cequal) &
		   (branch
			 (quote (## 1))
			 (push & csnd &
			  swap & push & cfst  & csnd &
			  	 swap & push & csnd &
					swap & (quote (## 1)) &
					cons & (op cminus) &
				 cons & app &
			  cons & (op ctimes)))))) &
	cons &
		push & csnd &
		swap & (quote (## N)) &
		cons & app)).


%-----------------------------------------------------------------------%

%%% CAM code for the even/odd program in Kahn's paper %%%

evenprog N 
 (program 
   (push & 
    push & 
    (rec (push & 
        (cur (push & (push & csnd & swap & quote (## 0) & cons & op cequal) &
	      branch (quote ctt) 
		     (push & (push & (cfst & csnd) & csnd) & swap & 
		       (push & csnd & swap & quote (## 1) & cons & op cminus) &
		       cons & app))) & 
	 swap & 
	 (cur (push & (push & csnd & swap & quote (## 0) & cons & op cequal) &
	      branch (quote cff) 
		     (push & (push & (cfst & csnd) & cfst) & swap & 
		       (push & csnd & swap & quote (## 1) & cons & op cminus) &
		       cons & app))) & 
	 cons)) &
    cons & 
    (push & (push & csnd & cfst) & 
    swap & 
    quote (## N) & 
    cons & app))).


%-----------------------------------------------------------------------%

%%% CAM code for appending [1] and [2] %%%

appendprog
  (program
    (push & 
     (push & 
      (push & 
       rec (cur 
            (cur (push & 
                  (push & cfst & csnd & swap & quote cnil & cons & op cequal) &
		  branch csnd (push & ((cfst & csnd) & op chd) & swap & 
			        (push & (push & (cfst & (cfst & csnd)) & swap &
			        ((cfst & csnd) & op ctl) & cons & app) &
				swap & csnd & cons & app) & 
				cons & op ccons))))) &
    swap & (push & quote (## 1) & swap & quote cnil & cons & op ccons) &
	    cons & app) & 
    swap & (push & quote (## 2) & swap & quote cnil & cons & op ccons) &
	    cons & app)).


%-----------------------------------------------------------------------%

%%% GCD Program (computes GCD of M and N) %%%

gcdprog M N (program 
(push & (push & (push & 
rec (cur 
    (cur (push & (push & (cfst & csnd) & swap & 
                  csnd & cons & op cequal) & 
                 branch (cfst & csnd) 
			(push & (push & (cfst & csnd) & swap & 
				 csnd & cons & op clt) & 
			 branch (push & (push & (cfst & (cfst & csnd)) & swap &
				  (push & csnd & swap & (cfst & csnd) & cons &
				    op cminus) & 
				  cons & app) & swap & 
				  (cfst & csnd) & cons & app)
				(push & (push & (cfst & (cfst & csnd)) & swap &
				 csnd & cons & app) & swap & 
				 (push & (cfst & csnd) & swap & csnd & cons &
				   op cminus) & 
				 cons & app)))))) & 
    swap & quote (## M) & cons & app) & 
    swap & quote (## N) & cons & app)).


%-----------------------------------------------------------------------%

%%% compute the length of the list [7] %%%

lenprog (program 
(push & (push & 
rec (cur 
     (push & (push & csnd & swap & quote cnil & cons & op cequal) & 
	     branch (quote (## 0)) 
		    (push & (push & (cfst & csnd) & swap & 
			            (csnd & op ctl) & cons & app) &
                            swap & quote (## 1) & cons & op cplus)))) & 
    swap & (push & quote (## 7) & 
    swap & quote cnil & cons & op ccons) & 
  cons & app)).


%-----------------------------------------------------------------------%

%%% Map the function "+1" over the list [1,2] %%%

mapprog (program 
(push & (push & (push & 
  rec (cur 
      (cur (push & 
             (push & csnd & swap & quote cnil & cons & op cequal) & 
	     branch (quote cnil) 
		    (push & 
		      (push & (cfst & csnd) & swap & 
			      (csnd & op chd) & cons & app) & 
		      swap & (push & 
				(push & (cfst & (cfst & csnd)) & swap & 
				  (cfst & csnd) & cons & app) & swap & 
				(csnd & op ctl) & cons & app) & 
		      cons & op ccons))))) & 
  swap & cur (push & csnd & swap & quote (## 1) & cons & op cplus) & 
    cons & app) & 
  swap & (push & quote (## 1) & 
  swap & (push & quote (## 2) & swap & quote cnil & 
    cons & op ccons) & 
    cons & op ccons) & 
  cons & app)).




%-----------------------------------------------------------------------%
%-- Exercises for the CAM code:
%-----------------------------------------------------------------------%


%%% Compute the factorial of N and return answer in V %%%
camfact N V :- 
	factprog N C,
	exec C V.

%%% Compute whether N is even and return answer in V %%%
cameven N V :-
	evenprog N C,
	exec C V.

%%% Append [1] and [2] %%%
camapp V :-
	appendprog A,
	exec A V.

%%% Gcd of M and N (try (2,4) for example) %%%
camgcd M N V :-
	gcdprog M N P,
	exec P V.

%%% Length of [7] %%%
camlen V :-
	lenprog P,
	exec P V.

%%% Map +1 over [1,2] %%%
cammap V :-
	mapprog P,
	exec P V.