%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%	CAM : Categorical Abstract Machine		  %%%
%%%	Test Cases for translator			  %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

module cmltest.

import mldecl camdecl camml.

p1 (# 3).
p2 tt.
p3 (cond tt (# 3) (# 4)).
p4 ((# 3) && tt).
p5 (let Id\Id (# 3)).
p6 (lamb Id\Id).
p7 ((lamb Id\Id) @ (# 3)).
p8 (== @ (# 3) @ (# 3)).
p9 (plus @ (# 3) @ (# 4)).
p10 (minus @ (# 3) @ (# 4)).
p11 (times @ (# 3) @ (# 4)).
p12 (:: @ (# 1) @ nil).
p13 (hd @ (:: @ (# 1) @ nil)).
p14 (tl @ (:: @ (# 1) @ nil)).
p15 (empty @ (:: @ (# 1) @ nil)).
p16 (empty @ nil).

%%% factorial in mini-ML %%%
fact1 (fix Fact\ (lamb N\ (cond (== @ N @ (# 0)) (# 1) 
				(times @ N @ (Fact @ (minus @ N @ (# 1))))))).

%%% mutual recursion example from Kahn's STACS paper: even/odd %%%
evenodd1 (fix EO\ 
	   ((lamb X\ (cond (== @ X @ (# 0)) 
			  tt 
			  ((snd @ EO) @ (minus @ X @ (# 1))))) &&
	    (lamb X\ (cond (== @ X @ (# 0)) 
			  ff 
			  ((fst @ EO) @ (minus @ X @ (# 1))))))).

%%% append function %%%
app1 (fix App\ (lamb K\ (lamb L\
                (cond (== @ K @ nil) 
                      L 
                      (:: @ (hd @ K) @ (App @ (tl @ K) @ L)))))).


%%% simple gcd function %%%
gcd1 (fix Gcd\ (lamb X\ (lamb Y\
		(cond (== @ X @ Y)
		      X
		      (cond (lt @ X @ Y)
			    (Gcd @ (minus @ Y @ X) @ X)
			    (Gcd @ Y @ (minus @ X @ Y))))))).

%%% length function %%%
len1 (fix Len\ (lamb L\
		(cond (== @ L @ nil)
		      (# 0)
		      (plus @ (Len @ (tl @ L)) @ (# 1))))).

%%% map function %%%
map1 (fix Map\ (lamb F\ (lamb L\
		(cond (== @ L @ nil)
		      nil
		      (:: @ (F @ (hd @ L)) @ (Map @ F @ (tl @ L))))))).


%---------------------------------------------------------------------------%


t1 C :- p1 E, translate E C.
t2 C :- p2 E, translate E C.
t3 C :- p3 E, translate E C.
t4 C :- p4 E, translate E C.
t5 C :- p5 E, translate E C.
t6 C :- p6 E, translate E C.
t7 C :- p7 E, translate E C.
t8 C :- p8 E, translate E C.
t9 C :- p9 E, translate E C.
t10 C :- p10 E, translate E C.
t11 C :- p11 E, translate E C.
t12 C :- p12 E, translate E C.
t13 C :- p13 E, translate E C.
t14 C :- p14 E, translate E C.
t15 C :- p15 E, translate E C.
t16 C :- p16 E, translate E C.
factcode C :- fact1 F, translate F C.
evenoddcode C :- evenodd1 F, translate (let EO\((fst @ EO) @ (# 3)) F) C.
appendcode C :- app1 A, 
	translate (A @ (:: @ (# 1) @ nil) @ (:: @ (# 2) @ nil)) C.
gcdcode C :- gcd1 G,  translate (G @ (# 2) @ (# 4)) C.

lencode C :- len1 L, translate (L @ (:: @ (# 7) @ nil)) C.

%%% map the function "+1" over the list [1,2] %%%
map1code C :- map1 M, translate (M @ (lamb N\(plus @ N @ (# 1)))
				   @ (:: @ (# 1) @ (:: @ (# 2) @ nil))) C.