%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%     mini-ML : Type Inference                          %%%
%%%     (infer Gamma Exp Type) succeeds if Exp has type   %%%
%%%     Type in "context" Gamma.                          %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

module mltype.

import mldecl lists.

type  infer   (list (pair tm tp)) -> tm -> tp -> o.


%%% Numbers in parentheses correspond to the numbering given to the
%%% rules in figure 3.

infer Gamma (# N) mint.                          %%% (1) Numbers

infer Gamma tt mbool.                            %%% (2,3) Boolean constants
infer Gamma ff mbool.

infer Gamma (cond E1 E2 E3) T :-                 %%% (4) Conditional
        infer Gamma E1 mbool,
        infer Gamma E2 T,
        infer Gamma E3 T.

infer Gamma (E1 && E2) (T1 mand T2) :-           %%% (5) Product Introduction
        infer Gamma E1 T1,
        infer Gamma E2 T2.

infer Gamma (lamb M) (T1 --> T2) :-              %%% (8) Abstraction
        pi C\(infer [(pair C T1)|Gamma] (M C) T2).

infer Gamma (E1 @ E2) T2 :-                      %%% (9) Application
        infer Gamma E1 (T1 --> T2),
        infer Gamma E2 T1.

infer Gamma (let M E2) T1 :-                     %%% (10) Let
        infer Gamma E2 T2,
        infer Gamma (M E2) T1.

infer Gamma  (fix M) T :-                        %%% (11) Recursion
        pi C\(infer [(pair C T)|Gamma] (M C) T).


%%% primitive operators %%%

infer Gamma fst ((T1 mand T2) --> T1).           %%% (6) First Projection
infer Gamma snd ((T1 mand T2) --> T2).           %%% (7) Second Projection

infer Gamma :: (T --> (mlist T) --> (mlist T)).  %%% list constructor
infer Gamma hd ((mlist T) --> T).                %%% head of list
infer Gamma tl ((mlist T) --> (mlist T)).        %%% tail of list
infer Gamma nil (mlist T).                       %%% the empty list
infer Gamma empty ((mlist T) --> mbool).         %%% empty-list function

infer Gamma == (T --> T --> mbool).              %%% term equality
infer Gamma != (T --> T --> mbool).              %%% term inequality
infer Gamma lt (mint --> mint --> mbool).        %%% integer <
infer Gamma gt (mint --> mint --> mbool).        %%% integer >
infer Gamma plus (mint --> mint --> mint).       %%% integer +
infer Gamma minus (mint --> mint --> mint).      %%% integer -
infer Gamma times (mint --> mint --> mint).      %%% integer *


%%% Assumed Types (instead of `=>') %%%

infer Gamma C T :-
        member (pair C T) Gamma.

%%%   End of module mltype   %%%

