% A meta-interpreter for LambdaProlog without higher-order predicates.
% This handles the most general case for implication in programs.
% It does not interpret cut or special predicates.
% Author: Frank Pfenning (fp@cs.cmu.edu)

module meta.

import meta_types.

type  solve         prop -> prop -> o.
type  match_clause  prop -> prop -> prop -> o.

solve ltrue PP :- !.
solve (G1 land G2) PP :- !, solve G1 PP, solve G2 PP.
solve (G1 lor G2) PP :- !, (solve G1 PP; solve G2 PP).
solve (P limp G) PP :- !, solve G (P land PP).
solve (lforall G) PP :- !, pi X\ (solve (G X) PP).
solve (lexists G) PP :- !, solve (G X) PP.
solve Atom PP :- !, match_clause Atom PP Subgoal,
		    solve Subgoal PP.

match_clause Atom (P1 land P2) Subgoal :- !,
    (match_clause Atom P1 Subgoal;
     match_clause Atom P2 Subgoal).

match_clause Atom (lforall P) Subgoal :- !,
    match_clause Atom (P X) Subgoal.

match_clause Atom (G limp P) (G land Subgoal) :- !,
    match_clause Atom P Subgoal.

match_clause Atom Atom ltrue.
