%  The clauses for removing patterns by explicit uses of data type destructors

module rem_pattern.

import solve_pat.

type	trans_bcl	((A1 -> A2) -> A1 -> assignment) -> 
				(A1 -> bool) -> ((A1 -> A2) -> A1 -> A2) -> o.
type	trans_blist	((A1 -> A2) -> A1 -> (list assignment)) -> 
						((A1 -> A2) -> A1 -> A2) -> o.
type	trans_pat	A1 -> ((A2 -> A3) -> A1) -> (A2 -> A3) -> o.
type	trans_mono	(A1 -> A2) -> (A1 -> A2) -> o.
type	trans_bin	(A1 -> A2 -> A3) -> (A1 -> A2 -> A3) -> o.
type	curry		((pair A1 A2) -> A3) -> (A1 -> (A2 -> A3)) -> o.



% From a binding clause extract a condition that is satisfied by the input
% when it matches the pattern and a functional form for forming the output

% The first step is to replace the pattern variable by a function of the input
trans_bcl Fun\I\(local X\ (B Fun I X)) Cond Body :-
		trans_bcl Fun\I\ (B Fun I (XX I)) Cond Body.
% Now use force_pat to find the condition
trans_bcl Fun\I\((Fun (F I)) -->  (C I)) Cond (Q\C) :-
					force_pat F Cond.
% Some care in forming the functional form is needed when the output involves 
% recursion, since the recursion must be over ALL binding clauses
trans_bcl Fun\I\((Fun (F I)) --> (C0 I (Fun (C1 I))))
          Cond (Q\X\ (C0 X (Q (C1 X)))) :-
					force_pat F Cond.


% Transforming a `list' of binding clauses into a functional program without 
% patterns. This uses trans_bcl, and its logic is obvious.
trans_blist (Fun\I\ []) (Q\X\error).
trans_blist (Fun\I\ [(First Fun I)|(Rest Fun I)])
            (Q\X\ (cond (Cond X) (Body Q X) (RestFp Q X))) :-
			trans_bcl First Cond Body, trans_blist Rest RestFp.



% An `interface' to trans_blist. trans_pat essentially strips off fixpt 
% and usepat and inserts an abstraction over the input variable at the 
% input end, and reinserts a fixpt at the output end. The extra argument is 
% for converting uncurried versions to curried versions.
trans_pat (fixpt (Fun\ (usepat (Fp_pat Fun)))) 
          CallingForm (fixpt Fp_sans) :- 
		trans_blist (Fun\I\ (Fp_pat (CallingForm Fun))) Fp_sans.


% Use trans_mono if the function defined is a one argument one
trans_mono Fp_pat Fp :- trans_pat Fp_pat (F\F) Fp.

% Use this if the function is in curried form. Note the instantiation 
% of CallingForm in trans_pat
trans_bin  Fp_pat Fp :- trans_pat Fp_pat (F\X\Y\ (F (pair X Y))) Uncurried,
                        curry Uncurried Fp.

% The currying transformation, i.e. convert a recursive function over a pair 
% of arguments to a recursive function of two arguments.
curry (fixpt (Q1\X\ (A (R1\R2\ (Q1 (pair R1 R2))) (fst X) (snd X) (pairp X))))
      (fixpt (Q2\Y1\Y2\ (A Q2 Y1 Y2 truth))).

