% One example file for all the different type
% inference modules.
% For the sake of conciseness of the answer, the second-order type
% variables Ftp and Atp are made explicit.  The simpler method
% of calling 'termtype' would be with its second argument uninstantiated,
% but the output even harder to interpret.

module tp_ex.

import tp_simple tp_types tp_polytypes.

% First example: the Church numeral for 3.

exs0 Ftp Xtp Alpha :- 
    termtype (olam    F\ (olam    X\ (F @ (F @ (F @ X)))))
	       (lam Ftp F\ (lam Xtp X\ (F @ (F @ (F @ X)))))
	       Alpha.

% Next example fails to terminate: unification is not smart enough to
% realize non-unifiability.

exs1 Ftp Alpha :-
     termtype (olam    F\ (F @ F))
		(lam Ftp F\ (F @ F))
		Alpha.

% Now examples for generic let.
% exm0 and exm1 should succeed,
% exm2 is an equivalent formulation using abstraction and application
%  and should fail.
% There is a possible problem here, namely that the language extension
%  with let and # are defined in tp_ml and tp_scon and therefore are
%  not available at parse time.

exm0 Xtp Alpha :-
   tp_ml => tp_scon =>
     termtype
        (tlet (F\ (tpair @ (F @ (# 1)) @ (F @ ttrue)))
	      (olam    X\ X))
        (tlet (F\ (tpair @ (F @ (# 1)) @ (F @ ttrue)))
	      (lam Xtp X\ X))
	Alpha.

exm1 Gtp Xtp Alpha :-
   tp_ml => tp_scon =>
     termtype
        (tlet (F\ (olam    G\ (G @ (F @ (# 1)) @ (F @ ttrue))))
	      (olam    X\ X))
        (tlet (F\ (lam Gtp G\ (G @ (F @ (# 1)) @ (F @ ttrue))))
	      (lam Xtp X\ X))
	Alpha.

exm2 Gtp Xtp Alpha :-
   tp_ml => tp_scon =>
     termtype
        (  (olam    (F\ (olam    G\ (G @ (F @ (# 1)) @ (F @ ttrue)))))
         @ (olam    X\ X)) 
        (  (lam Ftp (F\ (lam Gtp G\ (G @ (F @ (# 1)) @ (F @ ttrue)))))
         @ (lam Xtp X\ X))
	Alpha.

% Now examples for polymorphic type inference.

% exp0 Should have 3 most general solutions.
exp0 Ftp Atp Alpha :-
   tp_poly =>
     termtype
       (ll A\ (olam        (F\ (lam A (X\ ((F &&)      @ X))))))
       (ll A\ (lam (Ftp A) (F\ (lam A (X\ (F & (Atp A) @ X))))))
       Alpha.

% This should not have any solution.  Test for graceful failure.
exp1 Ftp Atp Alpha :-
   tp_poly =>
     termtype
       (ll A\ (olam    (F\ (lam A (X\ ((F &&)    @ X))))))
       (ll A\ (lam Ftp (F\ (lam A (X\ (F & Atp @ X))))))
       Alpha.

% exp2 does not seem to return after the first solution is found.
exp2 Ftp Atp1 Atp2 Alpha :-
   tp_poly => tp_scon =>
     termtype
       (olam    (F\ (ll A\ (lam A (X\
          ((F &&)       @ (# 1) @ ((F &&)       @ X @ X)))))))
       (lam Ftp (F\ (ll A\ (lam A (X\
          (F & (Atp1 A) @ (# 1) @ (F & (Atp2 A) @ X @ X)))))))
       Alpha.

% This should have two most general solution, one of them
% containing the absurd type.
% This will only terminate after a
% ?- switch projfirst on.
% and even then one must type '.' after the second solution.
exp3 Xtp Atp Alpha :-
   tp_poly =>
     termtype
       (olam    (X\ ((X &)    @ X)))
       (lam Xtp (X\ ((X & Atp) @ X)))
       Alpha.
