
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                            FILE   MODULES.PRO                               %
%                          ----------------------                             %
%                                                                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


 load_modules([]).

 load_modules([Mname|Mlist]) :- 
           (module_list(C);C=[]), member(Mname,C),!,
           write_mod_name(Mname), write(' -- already loaded'), nl,
           load_modules(Mlist).

 load_modules([Mname|Mlist]) :-
           write_mod_name(Mname), 
           (make_file_name(Mname,'.mod',Fname), !, nl,
                  inctab, open_file(OFile,Fname), load_module(Mname), 
                  close_file(OFile), dectab ;
            write(' -- cannot be loaded; file nonexistent'), nl),
           load_modules(Mlist).




load_module(Mname) :-
           (retract(module_list(Mlist)), ! ; Mlist = []), 
                                    assert(module_list([Mname|Mlist])),
           (system_module(Mname) ;
            assert(import(Mname,[sys_defs])), assertz(mlink(Mname,sys_defs))),
           !,
           read_line(L1), parse_modname(Mname,L1,L1,[]),
           read_line(L2), parse_modbody(Mname,import_dec,L2).


write_mod_name(Mname) :- write_tab, write(Mname).

make_file_name(Filename,Ext,Fullname) :- 
	path_name(Path), append_names(Path,Filename,Partname), 
	append_names(Partname,Ext,Fullname), file_exists(Fullname).



delete_from_importlist(Mname,Modlist) :-
              (retract(import(Mname,Modl)) ; Modl = []),
              delete_list(Modlist,Modl,NModl), assert(import(Mname,NModl)).

delete_mlinks(M,[]).
delete_mlinks(M,[N|L]) :-
     (retract(mlink(M,N));true), delete_mlinks(M,L).



% The following two clauses work in tandem to provide stacking of files 
open_file(OFile,NFile) :-
   seeing(OFile), 
   see(NFile), nofileerrors, blank(Char), assert(look_ahead(Char)).

close_file(OFile) :- seen, see(OFile).

unload_module(Mname) :-
  retractall(import(Mname,Mlist)),
  retractall(operator(Mname,Fix,Priority,Op,Mode)),
  retractall(kind(Mname,Tok,Kind)),
  retractall(type(Mname,Tok,Type)),
  retractall(expression(Mname,Tok,Exp)),
  retractall(lp_clause(Mname,Pred,Clause)),
  retractall(mlink(Mname,_)),
  (retract(module_list(Mlist));Mlist=[]),
  remove_elt(Mname,Mlist,NewMlist),
  assert(module_list(NewMlist)).

reload(Module) :-
  unload_module(Module),
  load_module(Module).

unload_all :- 
  retractall(import(_,_)),
  retractall(infix(_,_,_,_,_)),
  retractall(kind(_,_,_)),
  retractall(type(_,_,_)),
  retractall(expression(_,_,_)),
  retractall(lp_clause(_,_,_)),
  retractall(module_list(_)),
  retractall(mlink(_,_)).

