%
%  This file contains operator and type declarations 
%  which are built into the Lambda Prolog interpreter.
%  
% This is a system module.  Do not import this one directly.

module lp_sys.

infix	255	:-	xfy.  
infix	254	;	xfy.  
infix	253	,	xfy.  % change prec of '|' in parser.pro with this
infix	252	=>	xfy.
infix	40	=	xfx.

infix	250	is	xfx.  
infix	250	iss	xfx.  


kind	int	type.         % change integertype and stringtype in 
kind	string	type.         % initial.pro if these names are changed.
kind	o	type.         
kind	opt	type.
kind 	swopt	type.
kind	onoff	type.

% For entering either lp or fp from Prolog
type		lp		o.
type		fp		o.


% First the logical connectives.
type		true		o. 
type		fail		o. 
type		!		o. 
type		clean_ff	o. 
type		not		o -> o. 
type		:-		o -> o -> o.
type		=		A -> A -> o.
type		,		o -> o -> o.
type		;		o -> o -> o.
type		=>		A -> o -> o.
type		sigma		(A -> o) -> o.   
type		pi		(A -> o) -> o. 
type		is		A -> A -> o.  
type		iss		(A -> o) -> (A -> o) -> o. 

% Constants used in tracing. The tracing functions and the options.
type		trace		opt -> o.
type		untrace		opt -> o.
type		clause		opt.
type		goal		opt.
type		interp		opt.
type		unifier		opt.
type		critical	opt.
type		allopts		opt.

% Constants used in setting switches
type		switch		swopt -> onoff -> o.
type		printtypes	swopt.
type		freezetypes	swopt.
type		projfirst	swopt.
type		tvw		swopt.
type		on		onoff.
type		off		onoff.

% Constants used in interacting with modules. A must be a variable in first two
type		paths		A -> o.
type		add_path	A -> o.
type		remove_path	A -> o.
type		use		A -> o.
type		unuse		A -> o.
type		load		A -> o.
type		reload		A -> o.
type		unload		A -> o.
type		write_module	A -> o.

% Constants used in implementing the help functions
type		help		o.
type		describe	A -> o.
type		describe_=	o.
type		describe_is	o.
type		describe_iss	o.

% Constants used to implement input/output
type		read		A -> o. 
type		see		A -> o.  % change A to file? 
type		seen		o.      
type		write		A -> o. 
type		writesans	string -> o. 
type		tell		A -> o.  % change A to file? 
type		told		o. 
type		nl		o. 
type		writeblank	o.   % These two should be removed from

% Constants for seeing the `internal' states
type		listing		Pred -> o.  
type		definition 	A -> o -> o.
type		internal	A -> o.

% For defining functions to be used in the functional programming
% interpreter.
type		defun		A -> A -> o.   

% Constants mainly used in debugging an lp implementation
type		recon		A -> o.

% Constants for implementing functions to exit lp
type		halt		o.
type		stop		o.
type		break		o.
type		bye		o.

% Miscellaneous functions
type		save		o.
type		statistics	o.
type		clean_ff	o.
